#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "struct.h"
#include "data.h"
#include "paths.h"
#include "aliens.h"
#include "player.h"
#include "torps.h"
#include "point.h"
#include "explosions.h"
#include "prize.h"
#include "blood.h"

/* #define convoy_x_pos(i) (convoyx+(20 * (i - 10 * (i/10))))*/
#define convoy_x_pos(i) (convoyx+(20 * (i%10)))
#define convoy_y_pos(i) (20 + (20*(i/10)))

struct torp *first_etorp=0;
double convoyx = 0, convoymove = 1;
int livecount = 0;
int attacking = 0, maxattacking, entering=0;
int maxetorps = 5, numetorps=0;
int killed_aliens=0;
struct alien aliens[MAXALIENS];

double moves[16][2] =
{
  { 0,-4},
  { 1,-4},
  { 3,-3},
  { 4,-1},
  { 4, 0},
  { 4, 1},
  { 3, 3},
  { 1, 4},
  { 0, 4},
  {-1, 4},
  {-3, 3},
  {-4, 1},
  {-4, 0},
  {-4,-1},
  {-3,-3},
  {-1,-4}
};

void delete_etorps()
  {
    struct torp *tmp;

    while(first_etorp)
    {
      tmp = first_etorp->next;
      free(first_etorp);
      first_etorp = tmp;
    }
  }

void init_aliens(int level)
  {
    int i;

    convoyx=0;
    convoymove = 1;

    maxattacking = MAX_ATTACKING;
#if !defined(MORE_ATTACKING)
    if(maxattacking > 30)
    maxattacking = 30;
#endif /* MORE_ATTACKING */
    attacking = 0;
    maxetorps = MAX_ETORPS;
    numetorps = 0;

    delete_etorps();
    for(i=0;i<MAXALIENS;i++)
    {
      new_alien(level, i, &aliens[i]);
    }

#if defined(MORE_ALIENS)
    /*
      Leave 60+10*(level/10) aliens alive.
    */
    for(i=(ENTRY_DEFINED_ALIENS+MORE_ALIENS_PER_INC*(level/LEVELS_PER_INC));
        i<MAXALIENS;++i)
    {
      aliens[i].alive=0;
      aliens[i].state=DEAD;
    }
#endif /* defined(MORE_ALIENS)    */

#if !defined(MORE_FLAGSHIPS)    
    /*
      Knock all but 2 of the flagships.
    */
    aliens[0].state=aliens[0].alive = 0;
    aliens[1].state=aliens[1].alive = 0;
    aliens[2].state=aliens[2].alive = 0;
    aliens[4].state=aliens[4].alive = 0;
    aliens[5].state=aliens[5].alive = 0;
    aliens[7].state=aliens[7].alive = 0;
    aliens[8].state=aliens[8].alive = 0;
    aliens[9].state=aliens[9].alive = 0;

    /*
      Knock out the first 2 and last 2 guards.
    */
    aliens[10].state=aliens[10].alive = 0;
    aliens[11].state=aliens[11].alive = 0;
    aliens[18].state=aliens[18].alive = 0;
    aliens[19].state=aliens[19].alive = 0;

#else /*!defined(MORE_FLAGSHIPS)    */
    /*
      Knock most of the flagships, depending on level.
    */
    aliens[0].alive*=aliens[0].state=(level>=32);/* 10 flagships at level 32*/
    aliens[1].alive*=aliens[1].state=(level>=16);/* 8 flagships at level 16*/
    aliens[2].alive*=aliens[2].state=(level>=8); /* 6 flagships at level 8*/
    aliens[3].alive*=aliens[3].state=1;
    aliens[4].alive*=aliens[4].state=(level>=4); /* 4 flagships at level 4*/
    aliens[5].alive*=aliens[5].state=(level>=4);
    aliens[6].alive*=aliens[6].state=1;
    aliens[7].alive*=aliens[7].state=(level>=8);
    aliens[8].alive*=aliens[8].state=(level>=16);
    aliens[9].alive*=aliens[9].state=(level>=32);

    /*
      Knock some guards, depending on level
    */
    aliens[10].alive*=aliens[10].state=(level>=8);
    aliens[19].alive*=aliens[19].state=(level>=8);
    aliens[11].alive*=aliens[11].state=(level>=4);
    aliens[18].alive*=aliens[18].state=(level>=4);
    livecount=0;
    for(i=0;i<MAXALIENS;++i)
    {
      if(aliens[i].alive)
      {
        ++livecount;
      }
    }

#endif /*!defined(MORE_FLAGSHIPS)    */
    
    killed_aliens=0;
  }

static void alien_escorting(alien_t* escort_ap)
  {
    int fs = escort_ap->escorting;
#if defined(DEBUG_ALIEN_ESCORTING)
    fprintf(OUT_FILE, "Alien %i escorting flagship %i\n"
      , escort_ap->id, aliens[fs].id);
#endif /* defined(DEBUG_ALIEN_ESCORTING) */
    if(!aliens[fs].alive)
    {
      escort_ap->escorting = -1;
      escort_ap->state=ATTACKING;
    }
    else
    if(aliens[fs].state==ATTACKING)
    {
#if defined(DEBUG_ALIEN_ESCORTING)
      fprintf(OUT_FILE, "Alien %i escorting flagship %i setting dir to %i\n"
        , escort_ap->id, aliens[fs].id, aliens[fs].dir);
#endif /* defined(DEBUG_ALIEN_ESCORTING) */
      
      escort_ap->dir = aliens[fs].dir;
    }
    else
    {
      escort_ap->path = -1;
      escort_ap->steer = 2;
      escort_ap->escorting = -1;
      escort_ap->state=ATTACKING;
    }
  }

static int alien_can_attack(alien_t* alien_ap)
  {
    /*If it's a gaurd, don't let it attack on its own while flagships are still alive.*/
    if(alien_ap->type == GUARD_SHIP_1 || alien_ap->type == GUARD_SHIP_12)
    {
      int e_i;
      for(e_i= 0;e_i < FLAGSHIPS;++e_i)
      {
        if(aliens[e_i].alive)
        {
          return 0;
        }
      }
    }
    return ((entering == 0) && (attacking < maxattacking)
      && ((livecount < maxattacking)
        || ((get_random(10000)) < (2+level+2*(killed_aliens)))
    ) 
  ); 
  }

static void alien_in_convoy(alien_t* convoy_ap)
  {
    convoy_ap->location.x += convoymove;
    if(alien_can_attack(convoy_ap))
    /*
      (
      (entering == 0) &&
      (attacking < maxattacking) && 
      (
      (livecount < maxattacking) || 
      ((random()%10000) < (2+level+2*(killed_aliens)))
      ) && (convoy_ap->type != GUARD_SHIP_1 && convoy_ap->type != GUARD_SHIP_12)
      )
    */
    {
      switch(get_random(2))
      {
        case 0:
          path_dir(P_PEELLEFT, 0, &convoy_ap->dir, &convoy_ap->steer);
          convoy_ap->path = P_PEELLEFT;
          break;
        case 1:
          path_dir(P_PEELRIGHT, 0, &convoy_ap->dir, &convoy_ap->steer);
          convoy_ap->path = P_PEELRIGHT;
          break;
      }
      convoy_ap->path_pos = 0;
      convoy_ap->state=ATTACKING;
      attacking++;
      if(convoy_ap->type==FLAG_SHIP_0||convoy_ap->type==FLAG_SHIP_11)
      { /* Flagship, grab escorts */
        int escorts_i=0;
        int ei=0;
        for(ei=FLAGSHIPS;ei<MAXALIENS;++ei)
        {
          if(aliens[ei].alive)
          {
            if(aliens[ei].state==IN_CONVOY)
            {
              aliens[ei].escorting = convoy_ap->id;
              ++escorts_i;
              aliens[ei].state=ESCORTING;
              if(escorts_i>=3){break;}
            }
          }
        }
      }
    }
  }

static void new_etorp(point_t location, point_t speed)
  {
    struct torp *t;

    t = malloc(sizeof(struct torp));
    t->next = first_etorp;
    if(t->next)
    t->next->prev = t;
    t->prev = 0;
    first_etorp = t;

    t->location = location;
    t->speed = speed;
    t->alive = 1;
    t->frame = 0;
    numetorps++;
  }

void maybe_throw_etorp(const alien_t* alien_ap, const player_t* player_pp
  , int aim_level_i)
  {
    int tc=0;
    tc=TORPCHANCE-level/2-(int)(player_0.weapons[player_0.current_weapon] * 5);
    if(tc<WORST_TORPCHANCE){tc=WORST_TORPCHANCE;}
    //    printf("torpchance is %d\n", tc);
    if(numetorps<maxetorps&&(!(get_random(tc))))
    {
      //      printf("etorp fired\n");
      point_t miss_p_d = {0, 0};
      point_t miss_n_d = {0, 0};
    
      point_t aim_d = {0, 0};
      point_t speed_d = {0, 0};
      double mag_d=0;

      double etorp_speed_d=0;
    
      int etorp_miss_i=(ETORP_MISS-(alien_ap->type<<1))-(aim_level_i<<1);
    
      if(etorp_miss_i<playerShip->height*2){etorp_miss_i=playerShip->height*2;}
      if(etorp_miss_i<playerShip->width*2){etorp_miss_i=playerShip->width*2;}

      miss_p_d.x=get_random(etorp_miss_i);
      miss_n_d.x=-(get_random(etorp_miss_i));
      miss_p_d.y=get_random(etorp_miss_i);
      miss_n_d.y=-(get_random(etorp_miss_i));

      aim_d.x=miss_p_d.x+miss_n_d.x+player_pp->location.x;
      aim_d.y=miss_p_d.y+miss_n_d.y+player_pp->location.y;
      aim_d.x-=alien_ap->location.x;
      aim_d.y-=alien_ap->location.y;
      mag_d=(sqrt(aim_d.x*aim_d.x+aim_d.y*aim_d.y));
      etorp_speed_d=(ETORPSPEED+aim_level_i/5);
      speed_d.x=(aim_d.x/mag_d)*(etorp_speed_d);
      speed_d.y=(aim_d.y/mag_d)*(etorp_speed_d);
      new_etorp(alien_ap->location,speed_d);
    }
  }

static void alien_waiting_to_enter(alien_t* waiting_ap)
  {
    waiting_ap->enterdelay--;
#if defined(DEBUG_DO_ENTER)
    fprintf(OUT_FILE, "Alien %i delay %i before entry.\n", waiting_ap->id
      , waiting_ap->enterdelay);
#endif /* defined(DEBUG_DO_ENTER)        */
    if(waiting_ap->enterdelay<0)
    {
#if defined(DEBUG_DO_ENTER)
      fprintf(OUT_FILE, "Alien %i changing to ENTERING state.\n", waiting_ap->id);
#endif /* defined(DEBUG_DO_ENTER)        */
      waiting_ap->state=ENTERING;
    }
  }

static void alien_enter(alien_t* entering_ap)
  {
    if(entering_ap->path >= 0)
    {
#if defined(DEBUG_DO_ENTER)
      fprintf(OUT_FILE, "Alien %i entering, now at (%5.1f,%5.1f) .\n"
        , entering_ap->id, entering_ap->location.x, entering_ap->location.y);
#endif /* defined(DEBUG_DO_ENTER)        */
    
      entering_ap->steer--;
      if(!entering_ap->steer)
      {
        entering_ap->path_pos++;
        path_dir(entering_ap->path, entering_ap->path_pos, &entering_ap->dir
          , &entering_ap->steer);
#if defined(DEBUG_DIR)
        if(entering_ap->dir<0||entering_ap->dir>15)
        {
          fprintf(OUT_FILE
            , "Alien %i with path %i and path_pos %i got invalid dir %i !\n"
            , entering_ap->id, entering_ap->path, entering_ap->path_pos
            , entering_ap->dir);
          fprintf(OUT_FILE
            , "  Check array paths and function path_dir() in file paths.c for errors.");
        }
#endif /* defined(DEBUG_DIR)*/
        if(entering_ap->steer < 0)
        {
#if defined(DEBUG_DO_ENTER)
          fprintf(OUT_FILE
            , "Alien %i done entering, changing to RETURNING_TO_CONVOY state.\n"
            , entering_ap->state);
#endif /* defined(DEBUG_DO_ENTER)        */
          entering_ap->state=RETURNING_TO_CONVOY;
        }
        entering_ap->steer = 2*(entering_ap->steer/3);
      }
    }
  }

void collision_check_player(struct alien* crunch_ap,struct player* player_pp)
  {
#if defined(DEBUG_COLLISION_CHECK_PLAYER)
    fprintf(OUT_FILE
      , "Checking alien at (%5.1f,%5.1f) against player at (%5.1f,%5.1f).\n"
      , crunch_ap->location.x, crunch_ap->location.y, player_pp->location.x
      , player_pp->location.y);
#endif /*defined(DEBUG_COLLISION_CHECK_PLAYER)  */
    if(crunch_ap->state<=WAITING_TO_ENTER){return;}
    if((!player_pp->flash) && (!player_pp->dead) && (crunch_ap->alive)
      && ((crunch_ap->location.x)>(player_pp->location.x-(playerShip->width/2))
        && (crunch_ap->location.x)<(player_pp->location.x+(playerShip->width/2)))
      && ((crunch_ap->location.y)>(player_pp->location.y-(playerShip->height/2))
        && (crunch_ap->location.y)<(player_pp->location.y+(playerShip->height/2))))
    {
#if defined(DEBUG_COLLISION_CHECK_PLAYER)
      fprintf(OUT_FILE, "Alien at (%5.1f,%5.1f) hit player at (%5.1f,%5.1f).\n"
        , crunch_ap->location.x, crunch_ap->location.y, player_pp->location.x
        , player_pp->location.y);
#endif /*defined(DEBUG_COLLISION_CHECK_PLAYER)  */
      crunch_ap->dying=1;
      if(player_pp->shield)
      {
        torp_hit_alien(crunch_ap);
      }
      else if(player_0.safety_bubbles>0)
      {
        torp_hit_alien(crunch_ap);
        --player_0.safety_bubbles;
      }
      else
      {
        player_pp->dead=1;
      }
      new_explosion(player_pp->location, 2);
    }
  }

static void alien_moving(alien_t* mover_ap)
  {
    if(mover_ap->state==WAITING_TO_ENTER||mover_ap->state==IN_CONVOY)
    {
      return;
    }
    maybe_throw_etorp(mover_ap,&player_0,level);
    mover_ap->location.x += moves[mover_ap->dir][0]*1.5;
    mover_ap->location.y += moves[mover_ap->dir][1]*1.5;
    /*
      Since most of the aliens *start* out of bounds, don't do bounds
      checking on aliens that are still entering.
    */
    if(mover_ap->state==ENTERING)
    {
      return;
    }
    else if(mover_ap->location.y > SPACE_Y_MAX)
    {
      mover_ap->location.y = SPACE_Y_MIN;
      mover_ap->path = -1;
      mover_ap->steer = 2;
      mover_ap->escorting = -1;
      attacking--;
      mover_ap->state=RETURNING_TO_CONVOY;
      if(mover_ap->type==FLAG_SHIP_0||mover_ap->type==FLAG_SHIP_11)
      {
        int j=0;
        for(j=FLAGSHIPS;j<FLAGSHIPS_AND_GUARDS;j++)
        {
          if(aliens[j].escorting==mover_ap->id)
          {
            aliens[j].escorting = -1;
            aliens[j].state=RETURNING_TO_CONVOY;
          }
        }
      }
    }
    else if(mover_ap->location.y < 0)
    {
      mover_ap->dir = 8;
    }
    wrap_object(&mover_ap->location);
  }

static void alien_attacking(alien_t* attacker_ap)
  {
    attacker_ap->steer--;
    if(attacker_ap->steer <= 0)
    {
      if(attacker_ap->path >= 0)
      {
#if defined(DEBUG_DIR)
        int lastdir=attacker_ap->dir;
#endif /*defined(DEBUG_DIR)        */
                
        attacker_ap->path_pos++;
        path_dir(attacker_ap->path, attacker_ap->path_pos, &attacker_ap->dir, &attacker_ap->steer);

        /* Has the alien reached the end of its path?*/
        if(attacker_ap->steer < 0)
        {
#if defined(DEBUG_DIR)
          if(attacker_ap->dir<0||attacker_ap->dir>15)
          {
            fprintf(OUT_FILE
              , "Alien %i has invalid dir %i ! Resetting dir to last dir.\n"
              , attacker_ap->id, attacker_ap->dir);
            attacker_ap->dir=lastdir;
          }
#endif /*defined(DEBUG_DIR)        */

          do
          {
            switch(get_random(8))
            {
              case 0:
                start_path(P_LOOP, attacker_ap);
                break;
              case 1:
                start_path(P_SWOOP1, attacker_ap);
                break;
              case 2:
                start_path(P_SWOOP2, attacker_ap);
                break;
              case 3:
                start_path(P_ZIGZAG, attacker_ap);
                break;
              case 4:
                start_path(P_LOOP2, attacker_ap);
                break;
              case 5:
                start_path(P_SPIN, attacker_ap);
                break;
              case 6:
                start_path(P_LEFTDIAG, attacker_ap);
                break;
              case 7:
                start_path(P_RIGHTDIAG, attacker_ap);
                break;
              default:
                attacker_ap->steer = TURNSPEED;
                attacker_ap->path = -1;
            }
          }while((attacker_ap->path < 0) || (attacker_ap->steer < 0)) ;
        }
      }
      else
      {
        if(get_random(2))
        {
          attacker_ap->dir++;
          if(attacker_ap->dir > 15)
          {
            attacker_ap->dir = 0;
          }
        }
        else
        {
          attacker_ap->dir--;
          if(attacker_ap->dir < 0)
          {
            attacker_ap->dir = 15;
          }
        }
        attacker_ap->steer = TURNSPEED;
      }
    }
  }

static void alien_returning_to_convoy(alien_t* returner_ap)
  {
    double diffx=0;
    double diffy=0;
    diffx = ABS(convoy_x_pos(returner_ap->id) - returner_ap->location.x);
    diffy = ABS(convoy_y_pos(returner_ap->id) - returner_ap->location.y);
    if(diffy<5)
    {
      returner_ap->location.y = convoy_y_pos(returner_ap->id);
      if(diffx < 5)
      {
        returner_ap->location.x = convoy_x_pos(returner_ap->id);
        returner_ap->entering = 0;
        returner_ap->state=IN_CONVOY;
        returner_ap->dir=0;
        return;
      }
      returner_ap->dir = 0;
      if(convoy_x_pos(returner_ap->id) > returner_ap->location.x)
      {
        returner_ap->dir = 4;
      }
      else
      {
        returner_ap->dir = 12;
      }
    }
    else
    {
      if(diffx < 5)
      {
        returner_ap->location.x = convoy_x_pos(returner_ap->id);
        if(returner_ap->location.y<convoy_y_pos(returner_ap->id))
        {
          returner_ap->dir = 8;
        }
        else
        {
          returner_ap->dir = 0;
        }
      }
      else
      {
        if(convoy_x_pos(returner_ap->id) > returner_ap->location.x)
        {
          if(returner_ap->location.y<convoy_y_pos(returner_ap->id))
          {
            returner_ap->dir = 10;
          }
          else
          {
            returner_ap->dir = 14;
          }
        }
        else
        {
          if(returner_ap->location.y<convoy_y_pos(returner_ap->id))
          {
            returner_ap->dir = 6;
          }
          else
          {
            returner_ap->dir = 2;
          }
        }
      }
    }
#if defined(DEBUG_DO_ENTER)
    fprintf(OUT_FILE, "Alien %i moving into convoy, now at (%5.1f,%5.1f) .\n"
      , returner_ap->id, returner_ap->location.x, returner_ap->location.y);
#endif /* defined(DEBUG_DO_ENTER)        */
    if((returner_ap->location.x < -4 * SPACE_WIDTH
         || returner_ap->location.x > 5 * SPACE_WIDTH)
      || (returner_ap->location.y < -4 * SPACE_HEIGHT
        || returner_ap->location.y > 5 * SPACE_HEIGHT))
    {
      fprintf(OUT_FILE,"  Alien %i going to hell!\n",returner_ap->id);
      fprintf(OUT_FILE,"  Killing Alien %i at position (%5.1f,%5.1f).\n"
        , returner_ap->id,returner_ap->location.x,returner_ap->location.y);
      returner_ap->alive=0;
    }
  }

void do_aliens()
  {
    int i;

    /*
      Move the convoy.
    */
    if(!paused)
    {
      convoyx += convoymove;
      if(convoyx <= 0)
      {
        convoyx=0;
        convoymove = -convoymove;
      }
      else if(convoyx >= SPACE_WIDTH/2)
      {
        convoyx = SPACE_WIDTH/2;
        convoymove = -convoymove;
      }
    }
    
    livecount=0; attacking = 0;
    for(i=0, livecount=0, entering=0; i < MAXALIENS; i++)
    {
      if(aliens[i].alive)
      {
        livecount++;
        if(aliens[i].state==ATTACKING)
        {
          attacking++;
        }
        if(aliens[i].state==ENTERING)
        {
          entering++;
        }
      }
    }
#if defined(DEBUG_DO_ALIENS)
    fprintf(OUT_FILE,"%i living aliens.",livecount);
#endif /* defined(DEBUG_DO_ALIENS)        */

    for(i=0; i < MAXALIENS; i++)
    {
      if(aliens[i].alive)
      {
        if(!paused)
        {
          collision_check_player(&aliens[i],&player_0);

          alien_moving(aliens+i);
                
          switch(aliens[i].state)
          {
            case WAITING_TO_ENTER:
              alien_waiting_to_enter(aliens+i);
              break;
            case ENTERING:
              alien_enter(aliens+i);
              break;
            case IN_CONVOY:
              alien_in_convoy(aliens+i);
              break;
            case ATTACKING:
              alien_attacking(aliens+i);
              break;
            case ESCORTING:
              alien_escorting(aliens+i);
              break;
            case RETURNING_TO_CONVOY:
              alien_returning_to_convoy(aliens+i);
              break;
            case DEAD:
              /* It's dead. What do you expect me to do? Resurrect it?*/
              break;
            default:
              fprintf(OUT_FILE,"Alien %i in unknown state %i!\n",i,aliens[i].state);
          }
#if defined(DEBUG_DIR)
          if((aliens[i].dir<0)||(aliens[i].dir>15))
          {
            fprintf(OUT_FILE,"Alien %i with invalid dir %i! Resetting to 0."
              , aliens[i].id, aliens[i].dir);
          }
#endif /* defined(DEBUG_DIR)*/


        }
      }
      if(aliens[i].dying)
      {
        aliens[i].alive = 0;
        aliens[i].dying=0;
        aliens[i].state=DEAD;
      }
    }
  }

void torp_hit_alien(alien_t* victim_ap)
  {
    score_alien_hit(victim_ap);
    if(victim_ap->alive>1)
    {
      --(victim_ap->alive);
      return;
    }
    ++killed_aliens;
    victim_ap->dying=1;
    switch(victim_ap->state)
    {
      case WAITING_TO_ENTER:
        fprintf(OUT_FILE
          , "Error! torp_hit_alien() called on alien %i in state WAITING_TO_ENTER !\n"
          , victim_ap->id);
        break;
      case ENTERING:
        --entering;
        break;
      case IN_CONVOY:
        /* Do nothing.*/
        break;
      case ATTACKING:
        --attacking;
        break;
      case ESCORTING:
        /* Do nothing. */
        break;
      case RETURNING_TO_CONVOY:
        /* Do nothing. */
        break;
      case DEAD:
        fprintf(OUT_FILE
          , "Error! torp_hit_alien() called on alien %i in state DEAD !\n"
          , victim_ap->id);
        break;
      default:
        fprintf(OUT_FILE
          , "Error! torp_hit_alien() called on alien %i in unknown state !\n"
          , victim_ap->id);
    }
    if(blood_on)
    {
      create_blood(victim_ap->location, victim_ap->type);
    }
    victim_ap->state=DEAD;
#if defined(DEBUG_DIR)
    if(victim_ap->dir < 0 || victim_ap->dir > 15)
    {
      fprintf(OUT_FILE,"Alien %i with invalid dir %i! Resetting to 0."
        , victim_ap->id, victim_ap->dir);
    }
#endif /* defined(DEBUG_DIR) */
  }

void score_alien_hit(const alien_t* victim_ap)
  {
    double prize_chance= PRIZECHANCE;
    int score_alien= 0;
    switch(victim_ap->type)
    {
      case FLAG_SHIP_0:
      case FLAG_SHIP_11:
        score_flag_ship_hit(victim_ap);
        new_explosion(victim_ap->location,1);
        break;
      default:
        new_explosion(victim_ap->location,0);
        {
          int mul_i=10;
          switch(victim_ap->state)
          {
            /* WAITING_TO_ENTER, DEAD, and default cases all report errors.  Also
               they are checked by torp_hit_alien.  As was, any error would be
               reported twice. */
            case WAITING_TO_ENTER:
              /* Do nothing.*/
              break;
            case ENTERING:
              mul_i<<=1;
              break;
            case IN_CONVOY:
              /* Do nothing.*/
              break;
            case ATTACKING:
              mul_i<<=2;
              break;
            case ESCORTING:
              mul_i<<=2;
              break;
            case RETURNING_TO_CONVOY:
              mul_i<<=1;
              break;
            case DEAD:
              /* Do nothing.*/
              break;
            default:
              /* Do nothing.*/
              break;
          }
          score_alien= mul_i*victim_ap->type;
          player_0.score+= score_alien;
          prize_chance= PRIZECHANCE - (PRIZECHANCE
            - ((PRIZECHANCE)/((score_alien/40.0) + level)))*0.8;
          if (player_0.got_lemon)
          {
            prize_chance/= 4 * player_0.got_lemon;
          }
          /*
          fprintf(OUT_FILE,"In %s prize_chance == %f, type == %i, level == %i.\n",
            __PRETTY_FUNCTION__, prize_chance, victim_ap->type, level);
          */
          if(!(get_random(((int)prize_chance) + 1)))
          {
            new_prize(victim_ap->location);
          }
        }
    }
  }

void score_flag_ship_hit(const alien_t* victim_ap)
  {
    double prize_chance= PRIZECHANCE;
    int escorts_i=0;
    int ki=0;

    for(ki=FLAGSHIPS;ki<MAXALIENS;++ki)
    {
      if(aliens[ki].escorting==victim_ap->id)
      {
        ++escorts_i;
      }
    }
    prize_chance= PRIZECHANCE - (PRIZECHANCE
      - ((PRIZECHANCE)/(victim_ap->type*2.0 + level + escorts_i)))*0.8;
    if (player_0.got_lemon)
    {
      prize_chance/= 4;
    }
    /*
    fprintf(OUT_FILE,"In %s prize_chance == %f, type == %i, level == %i.\n",
      __PRETTY_FUNCTION__, prize_chance, victim_ap->type, level);
    */
    if(!(get_random(((int)prize_chance) + 1)))
    {
      new_prize(victim_ap->location);
    }
    score_flagship(victim_ap->location,escorts_i);
  }

void do_etorps()
  {
    struct torp *t = first_etorp, *nextt;
    int i;

    i= 0;
    
    while(t)
    {
      nextt=t->next;
      if(t->alive)
      {
        if(!paused)
        {
          t->location.y+=t->speed.y;
          t->location.x+=t->speed.x;
          t->frame++;
          if(player_0.shoot_down_etorps>0)
          {
            for (i= 0 ; i < MAXTORPS ; ++i)
            {
              /*Only collison check player torps that are alive.*/
              if(torps[i].alive > 0)
              {
                if((ABS(t->location.x - torps[i].location.x) < 4)
                  && (ABS(t->location.y - torps[i].location.y) < 4))
                {
                  t->alive = 0;
                  --torps[i].hits;
                  if(torps[i].hits<=0)
                  {
                    kill_torp(i);
                  }
                }
              }
            }
          }
          if(wrap_object(&t->location))
          {
            t->alive= 0;
          }
          else if(!player_0.dead && !player_0.flash && !player_0.shield
            && (ABS(t->location.x - player_0.location.x) < 8)
            && (ABS(t->location.y - player_0.location.y) < 8))
          { /* DEAD! */
            /*
              Check for safety bubbles.
              If the player has one or more safety bubbles,
              decrement the number of safety bubbles, and kill the
              etorp, but do
              not kill the player.
            */
            if(player_0.safety_bubbles>0)
            { /* safe! */
              --player_0.safety_bubbles;
              t->alive= 0;
            }
            else
            {
              player_0.dead = 1;
              new_explosion(player_0.location, 2);
              if(blood_on)
              {
                create_blood(player_0.location, player_0.ships * 10);
              }
              t->alive= 0;
            }
          }
        }
      }
      if (!t->alive)
      {
        if(t->next)
        {
          t->next->prev = t->prev;
        }
        if(t->prev)
        {
          t->prev->next = t->next;
        }
        if(t == first_etorp)
        {
          first_etorp = t->next;
        }
        free(t);
        if((!player_0.dead)&&(!player_0.shield)&&(!player_0.flash))
        {
          ++player_0.score;
        }
        numetorps--;
      }
      t = nextt;
    }
  }
