/* This contains all the draw functions.  All the logic is done elsewhere, so if you
   want to change the draw method, replace this file while leaving the function names
   the same. */
#include <stdio.h>
#include <string.h>

#include "libsprite/Wlib.h"
#include "struct.h"
#include "data.h"
#include "defs.h"
#include "explosions.h"
#include "player.h"
#include "blood.h"
#include "stars.h"
#include "aliens.h"
#include "score.h"
#include "torps.h"
#include "prize.h"
#include "title.h"

static int drewlevel = 0;

/*-wrappers for draw and clear functions--*/
void draw_image_wrapper(W_Window window, int x, int y, int frame, W_Image *image
  , W_Color color)
  {
    y += STATUS_BAR_HEIGHT;
    W_DrawImage(window, x, y, frame, image, color);
  }

void mask_text_wrapper(W_Window window, int x, int y, W_Color color, char *str
  , int len, W_Font font)
  {
    W_MaskText(window, x, y, color, str, len, font);
  }

void cache_clear_area_wrapper(W_Window window, int x, int y, int width, int height)
  {
    W_CacheClearArea(window, x, y, width, height);
  }

void clear_area_wrapper(W_Window window, int x, int y, unsigned int width
  , unsigned int height)
  {
    W_ClearArea(window, x, y, width, height);
  }

void prep_draw_area(void)
  {
    W_CacheClearArea(baseWin, 0, 0, WINWIDTH, WINHEIGHT);
    W_FlushClearAreaCache(baseWin);
  }

void display_draw_area(void)
  {
    W_DisplayBuffer(baseWin);
    W_Flush();
  }

void draw_frame(point_t object_location, point_t object_size, W_Color color)
  {
    point_t frame_location;
    point_t frame_size;
    frame_location.x= (object_location.x - (object_size.x / 2)) - 1;
    frame_location.y=((object_location.y-(object_size.y/2))+STATUS_BAR_HEIGHT)-1;
    frame_size.x= object_size.x + 2;
    frame_size.y= object_size.y + 2;
    W_FillArea(baseWin, frame_location.x, frame_location.y, frame_size.x
      , frame_size.y, color);
    frame_location.x++;
    frame_location.y++;
    frame_size.x-= 2;
    frame_size.y-= 2;
    W_FillArea(baseWin, frame_location.x, frame_location.y, frame_size.x
      , frame_size.y, W_Black);
  }

void draw_pause(void)
  {
    W_DrawImage(baseWin, pausex-(pauseImage->width/2), pausey-(pauseImage->height/2), 0, pauseImage, W_White);
  }

void draw_bubbles(void)
  {
    struct score_bubble *bub= first_bub, *nextbub;

    while(bub)
    {
      nextbub= bub->next;
      draw_image_wrapper(baseWin, ((int)bub->location.x)-(bub->shape->width/2)
        , ((int)bub->location.y)-(bub->shape->height/2), 0, bub->shape, W_Red);
      bub= nextbub;
    }
  }

void draw_explosions(void)
  {
    struct explosion *exp= first_exp, *nextexp;

    while(exp)
    {
      nextexp = exp->next;
      draw_image_wrapper(baseWin
        , ((int)exp->location.x)-(exp->shape->width/2)
        , ((int)exp->location.y)-(exp->shape->height/2)
        , exp->frame, exp->shape, W_Red);
      exp=nextexp;
    }
  }

void draw_blood(void)
  {
    int i;
    for(i= 0; i<1000; i++)
    {
      if(blood_dots[i].alive)
      {
        // printf("drawing blood_dots[%d].\n", i);
        W_FillArea(baseWin, blood_dots[i].location.x
          , blood_dots[i].location.y, BLOOD_SIZE, BLOOD_SIZE, blood_dots[i].color);
      }
    }
    for(i= 0; i<WINWIDTH; i++)
    {
      W_FillArea(baseWin, blood_pools[i].location.x, blood_pools[i].location.y
        , blood_pools[i].speed.x, blood_pools[i].speed.y, blood_pools[i].color);
    }
  }

void draw_stars(void)
  {
    int i;
    for(i= 0; i<NUMSTARS; i++)
    {
      W_DrawPoint(baseWin, stars[i].location.x, stars[i].location.y, stars[i].color);
    }
  }

void draw_level_announcement(void)
  {
    if(starspeed != 1)
    {
      char buf[20];
    
      drewlevel = 1;
      sprintf(buf, "LEVEL %d", level);
      mask_text_wrapper(baseWin, WINWIDTH/2 - (W_StringWidth(buf, W_BigFont)/2)
        , (WINHEIGHT/2-W_BigTextheight/2)+STATUS_BAR_HEIGHT,W_Red, buf, strlen(buf)
        , W_BigFont);
    }
  }

void draw_torps(void)
  {
    int i;
    for(i=0;i<MAXTORPS;i++)
    {
      if(torps[i].alive)
      {
          draw_image_wrapper(baseWin
            , (int)torps[i].location.x-(playerTorp->width/2)
            , (int)torps[i].location.y-(playerTorp->height/2)
            , torps[i].frame, playerTorp, W_Red);
      }
    }
  }

void draw_aliens(void)
  {
    int i;

    for(i= 0; i < MAXALIENS; i++)
    {
      if(aliens[i].alive)
      {
        draw_image_wrapper(baseWin
          , (int)aliens[i].location.x-(aliens[i].shape->width/2)
          , (int)aliens[i].location.y-(aliens[i].shape->height/2)
          , aliens[i].dir, aliens[i].shape, W_Green);
      }
    }
  }

void draw_etorps(void)
  {
    struct torp *t= first_etorp, *nextt;

    while(t)
    {
      nextt= t->next;
      draw_image_wrapper(baseWin
        , (int)t->location.x-(enemyTorp->width/2)
        , (int)t->location.y-(enemyTorp->height/2)
        , t->frame, enemyTorp, W_Red);
      t= nextt;
    }
  }

void draw_player(void)
  {
    if(!(player_0.flash % 2))
    {
      draw_image_wrapper(baseWin
        , player_0.location.x-(playerShip->width/2)
        , player_0.location.y-(playerShip->height/2)
        , counter, playerShip, W_Red);
    }
    if(player_0.shield && ((player_0.shield > SHIELDTIME/4) || player_0.shield % 2))
    {
      draw_image_wrapper(baseWin
        , (int)player_0.location.x-(shieldImage->width/2)
        , player_0.location.y-(playerShip->height/2) - 3
        , 0, shieldImage,W_Cyan);
    }
    else if(player_0.safety_bubbles)
    {
      draw_image_wrapper( baseWin
        , (int)player_0.location.x-(bubbleImage->width/2)
        , player_0.location.y-(playerShip->height/2) - 3
        , 0, bubbleImage,W_Cyan);      
    }
  }

void draw_score(void)
  {
    int basex;
    int i;

    /* Clear top stat area. */
    clear_area_wrapper(baseWin, 0, 0, WINWIDTH, STATUS_BAR_HEIGHT);
    basex = WINWIDTH/2 - ((strlen(scorestr)/2)*W_Textwidth);
    mask_text_wrapper(baseWin, basex, 0, W_Yellow, scorestr, strlen(scorestr)
      , W_RegularFont);
    basex = WINWIDTH/2 - ((strlen(scorestr2)/2)*W_Textwidth);
    mask_text_wrapper(baseWin, basex, 10, W_Yellow, scorestr2, strlen(scorestr2)
      , W_RegularFont);
    basex = WINWIDTH/2 - ((strlen(scorestr3)/2)*W_Textwidth);
    mask_text_wrapper(baseWin, basex, 20, W_Yellow, scorestr3, strlen(scorestr3)
      , W_RegularFont);
    basex = WINWIDTH/2 - ((strlen(scorestr4)/2)*W_Textwidth);
    mask_text_wrapper(baseWin, basex, 30, W_Yellow, scorestr4, strlen(scorestr4)
      , W_RegularFont);
    if(player_0.ships > 6)
    {
      basex = 6*(miniship->width+2) + 2;
      W_MaskText(baseWin, basex, 0, W_Yellow, shipstr, strlen(shipstr)
        , W_RegularFont);
    }
    if(drawExtra)
    {
	extra.x += 10;
	draw_image_wrapper(baseWin, extra.x - (extraImage->width / 2)
          , extra.y - (extraImage->height / 2), 0, extraImage, W_White);
	if((extra.x-(int)extraImage->width / 2) > WINWIDTH)
        {
	    drawExtra = 0;
        }
    }
    for(i=0;i<((player_0.ships < 6) ? player_0.ships : 6);i++)
    {
	W_DrawImage(baseWin, i*(miniship->width+2), 0, 0, miniship, W_White);
    }
  }

void draw_prizes(void)
  {
    prize_t *p= first_prize, *nextp;
    point_t size;
    while(p)
    {
      size.x= prizeImages[p->type]->width;
      size.y= prizeImages[p->type]->height;

      nextp= p->next;
      draw_frame(p->location, size, prize_frame_colors[p->category]);
      draw_image_wrapper(baseWin, ((int)p->location.x)-prizeImages[p->type]->width/2
        , ((int)p->location.y-prizeImages[p->type]->height/2),0, prizeImages[p->type]
        , W_Green);
      p= nextp;
    }
  }

void draw_output_string(void)
  {
    /* printf(".");
       fflush(stdout); */
    int basex= WINWIDTH/2 - ((strlen(output_string) / 2) * W_Textwidth);
    /* printf(".");
       fflush(stdout); */
    mask_text_wrapper(baseWin, basex, 100, W_Yellow, output_string
      , strlen(output_string), W_RegularFont);
    /* printf(".");
       fflush(stdout); */
  }
