#include <stdlib.h>
#include <stdio.h>

#include "defs.h"
#include "libsprite/Wlib.h"
#include "images.h"
#include "struct.h"
#include "data.h"
#include "sound.h"
#include "explosions.h"
#include "player.h"

struct explosion *first_exp=0;
struct score_bubble *first_bub=0;

W_Image *expImage, *bubbleImages[4];

void do_bubbles()
{
    struct score_bubble *bub=first_bub, *nextbub;

    while(bub)
    {
      nextbub = bub->next;
	if(!paused)
        {
	    bub->count--;
	    bub->location.y--;
	}
#if defined(DEBUG_DO_BUBBLES)
        fprintf(OUT_FILE,"Drawing score bubble at (%5.1f,%5.1f).\n",bub->x,bub->y);
#endif /* defined(DEBUG_DO_BUBBLES)        */
	if(bub->count <= 0)
        {
	    if(bub->prev)
		bub->prev->next = bub->next;
	    if(bub->next)
		bub->next->prev = bub->prev;
	    if(first_bub == bub)
		first_bub = 0;
	    free(bub);
	}
	bub=nextbub;
    }
}

void do_explosions()
{
    struct explosion *exp=first_exp, *nextexp;

    while(exp)
    {
      nextexp = exp->next;
      if(!paused) 
      exp->frame++;
      if(exp->frame >= exp->shape->frames)
      {
        if(exp->prev)
        exp->prev->next = exp->next;
        if(exp->next)
        exp->next->prev = exp->prev;
        if(first_exp == exp)
        first_exp = 0;
        free(exp);
      }
      exp=nextexp;
    }
}

void new_explosion(point_t location, int type)
{
    struct explosion *exp;

    exp = malloc(sizeof(struct explosion));
    exp->next = first_exp;
    exp->prev = 0;
    if(exp->next)
	exp->next->prev = exp;
    first_exp = exp;

    exp->location = location;
    exp->frame = 0;
    exp->shape = getImage(I_EXPLOSION);
#ifdef SOUND
    switch(type) {
      case 1:
	play_sound(SND_EXPLOSION);
	break;
      case 2:
	play_sound(SND_EXP_SB);
	break;
      case 0:
      default:
	play_sound(SND_TORPHIT);
	break;
    }
#endif
}


void score_flagship(point_t location, int ne)
{
  struct score_bubble *bub;
#if defined(DEBUG_SCORE_FLAGSHIP)
    fprintf
      (
        OUT_FILE,"Scoring flagship at (%5.1f,%5.1f) with %i escorts for %i points.\n",
        x,y,ne,(1<<ne)*500
      );
#endif /* defined(DEBUG_SCORE_FLAGSHIP)    */

    bub = malloc(sizeof(struct score_bubble));
    bub->next = first_bub;
    bub->prev = 0;
    if(bub->next)
	bub->next->prev = bub;
    first_bub = bub;

    player_0.score+=(1<<ne)*500;
    bub->location = location;
    bub->count=25;
    if(ne>3)
    {
      fprintf(OUT_FILE,"No score bubble for %i escorts.\n",ne);
      ne=3;
    }
    bub->shape = bubbleImages[ne];
}

void init_explosions()
{
    expImage = getImage(I_EXPLOSION);

    bubbleImages[0] = getImage(I_S500);
    bubbleImages[1] = getImage(I_S1000);
    bubbleImages[2] = getImage(I_S2000);
    bubbleImages[3] = getImage(I_S4000);
}
