/*
 * main.c for XGalaga
 * Copyright (c) 1995 Joe Rumsey
 */
#include "copyright.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <string.h>
#include <sys/time.h>

#include "libsprite/Wlib.h"
#include "data.h"
#include "main.h"
#include "images.h"
#include "defs.h"
#include "struct.h"
#include "paths.h"
#include "sound.h"
#include "aliens.h"
#include "stars.h"
#include "player.h"
#include "torps.h"
#include "blood.h"
#include "input.h"
#include "draw.h"
#include "explosions.h"
#include "prize.h"
#include "score.h"
#include "title.h"
#include "highscore.h"

void xgal_exit(int v)
  {
    /*    W_AutoRepeatOn();*/
#ifdef SOUND
    kill_sound();
#endif
    /* sleep(1);*/ /* Without this, the auto-repeat request fails on my machine... */
    exit(v);
  }


static void print_usage()
  {
    printf("\
  XGalaga v%s\n\
  Copyright (c) 1995 Joe Rumsey\n\
  <ogre@atomic.com>\n\
  Command line options:\n\
  -scores              Prints out the high score files and exits\n\
  -display <display>   Set your display\n\
  -mouse               Use mouse control (same as 'm' at the title screen)\n\
  -keyboard            Use keyboard control (same as 'k')\n%s\
  -b                   turn buffered mode off, use this if it runs\n\
  too slowly.  Will cause flicker, maybe lots,\n\
  maybe only a little.\n\
  \n\
  This game has the following copyrights:\n\
  Copyright (c) 1995   Joe Rumsey\n\
  Permission to use, copy, modify, and distribute this\n\
  software and its documentation for any purpose and without\n\
  fee is hereby granted, provided that the above copyright\n\
  notice appear in all copies and that both that copyright\n\
  notice and this permission notice appear in supporting\n\
  documentation.  No representations are made about the\n\
  suitability of this software for any purpose.  It is\n\
  provided \"as is\" without express or implied warranty.\n\
  (From copyright.h)\n\
  Copyright 1989		Kevin P. Smith\n\
  Scott Silvey\n\
  Permission to use, copy, modify, and distribute this\n\
  software and its documentation for any purpose and without\n\
  fee is hereby granted, provided that the above copyright\n\
  notice appear in all copies.\n\
  (From copyright2.h)\n\
  __LLeweLLyn__Reese__ added some hacks to create more aliens\n\
  as you rise in level.\n\
  So if this game is too hard for you, well, you can blame me.\n\
  Basic instructions:\n\
  It's Galaga, you know how to play Galaga, stop bothering me.\n\
  (Ship is moved by keys defaulted in player.c(can also be set\n\
  with -set_keys, space button fires.\n\
  Auto-fire by holding it down, so no-one accuses me of breaking\n\
  their keyboard!)\n\
  \n\
  Keyboard commands:\n\
  \n\
  p - pauses\n\
  q - end this game\n\
  b - Toggle buffering (flicker vs. speed.)\n\
  o - Toggle OR mode.  Don't use this, it sucks.\n", VERSION,
#ifdef SOUND
      "  -nosound             Turn sound OFF\n"
#else
      ""
#endif
  );
  }

void init_level()
  {
    init_torps();
    init_blood();
    init_aliens(level++);
  }

void check_level_end()
  {
    if(livecount == 0)
    {
      starspeed++;
#ifdef SOUND
      if(starspeed == 2)
      {
        play_sound(SND_WARP);
      }
#endif
      if(starspeed >= 120)
      {
        starspeed = -20;
      }
      else if(starspeed == 1)
      {
        init_level();
        player_0.got_lemon = 0;
        starspeed = 1;
      }
    }
  }

void set_keys()
  {
    int n;
    ev_t event;
    memcpy(player_0.keys, default_keys, sizeof(player_0.keys));
    printf("Setting keys...\n");
    /* First, eat all the keys already pressed so we start with an
       empty event list.*/
    for(n= 0; n < number_of_keys; n++)
    {
      clear_events();
      sprintf(output_string, "Press the key to bind to %s"
        , default_keys[n].description);
      prep_draw_area();
      draw_output_string();
      display_draw_area();
      event= wait_get_non_keyup_event();
      while(n > 0 && event.key == player_0.keys[n-1].chars)
      {
        event= wait_get_non_keyup_event();
      }
      player_0.keys[n].chars= event.key;
      printf("%s key is %c\n", default_keys[n].description, player_0.keys[n].chars);
    }
    printf("Keys set.\n");    
  }

void do_everything()
  {
    do_stars();
    if(blood_on)
    {
      do_blood();
    }
    do_explosions();
    do_bubbles();
    do_torps();
    do_prizes();
    do_aliens();
    do_etorps();
    do_player();
    if(paused)
    {
      do_pause();
    }
    do_score();
    check_level_end();
  }

void draw_everything()
  {
    /* Screw this running around undrawing each thing seperately!
       I'm undrawing the whole goddam window! */
    prep_draw_area();
    draw_stars();
    draw_level_announcement();
    if(blood_on)
    {
      draw_blood();
    }
    draw_explosions();
    draw_bubbles();
    draw_torps();
    draw_prizes();
    draw_aliens();
    draw_etorps();
    if(!gameOver && !player_0.dead)
    {
      draw_player();
    }
    if(paused && show_pause)
    {
      draw_pause();
    }
    if(gameOver)
    {
      do_title();
      if(getting_name)
      {
        do_name();
      }
    }
    draw_score();
    display_draw_area();
  }

int main(int argc, char** argv)
{
    int ac, set_keys_AFTER_init= 0;
    char *dpyname = 0;
    char* dis_en[2] = { "dis", "en"};
    struct timeval tv, otv;
    long ut;
   
    gettimeofday(&otv, 0);
    for(ac = 1; ac < argc; ac++)
    {
        if(*argv[ac] == '-')
        {
            if(strcmp(argv[ac], "-scores") == 0)
            {
                print_scores();
                exit(0);
            }
            if(strcmp(argv[ac], "-display") == 0 && (ac+1 < argc))
            {
                dpyname = argv[ac+1];
                ac++;
            }
            if(strcmp(argv[ac], "-set_keys") == 0)
            {
              set_keys_AFTER_init= 1;
              /* Don't fool with things that haven't been initialized.
                 It's bad for your health. */
            }
            else if(strcmp(argv[ac], "-pause") == 0)
            {
              show_pause= !show_pause;
              printf("Pause %sabled\n", dis_en[show_pause]);
            }
#if 0
            else if (strcmp(argv[ac], "-b") == 0)
            {
                useBuffered = !useBuffered;
            }
#endif
#ifdef SOUND
            else if (strcmp(argv[ac], "-nosound") == 0)
            {
                playSounds = 0;
            }
#endif
            else if (strcmp(argv[ac], "-blood") == 0)
            {
                 blood_on = !blood_on;
                 printf("Blood %sabled\n", dis_en[blood_on]);
            }
            else
            {
                print_usage();
                exit(0);
            }
        }
        else
        {
            print_usage();
            exit(0);
        }
    }
        
    W_Initialize(dpyname);

    backColor = W_Black;
    baseWin = W_MakeWindow("XGalaga", 0, 0, WINWIDTH, WINHEIGHT, 0, "tiny", 0, W_White);
    W_MapWindow(baseWin);

    /*    W_AutoRepeatOff();*/

    W_Flush();

    W_SetImageDir(IMAGEDIR);

    playerShip = getImage(I_PLAYER1);
    playerTorp = getImage(I_MTORP);
    enemyTorp = getImage(I_ETORP);
    shieldImage = getImage(I_SHIELD);
    bubbleImage= getImage(I_SAFETY_BUBBLE);

    level=1;

    if(set_keys_AFTER_init)
    {
      set_keys();
      keys_set= 1;
      printf("Keys set by set_keys() function.\n");
    }

    load_scores();
    init_titles();
    init_player(&player_0,level);
    init_stars();
    init_explosions();
    init_score();
    init_prizes();
#ifdef SOUND
    init_sound();
#endif
    init_level();

    while(1) {
        counter++;
        if(gameOver && getting_name)
        {
          //          clean_up_prizes();
        }
         
        do_everything();
        draw_everything();
        gettimeofday(&tv, 0);
        if(tv.tv_sec > otv.tv_sec)
            tv.tv_usec += 1000000;
        ut = tv.tv_usec - otv.tv_usec;
        if(UTIMER - ut > 0) {
            struct timeval tv;
            unsigned long usecs = UTIMER - ut;
            tv.tv_sec  = usecs / 1000000L;
            tv.tv_usec = usecs % 1000000L;
            (void) select (0, 0, 0, 0, &tv);
        }
        gettimeofday(&otv, 0);
    }
}
