#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

#include "libsprite/Wlib.h"
#include "libsprite/defs.h"
#include "images.h"
#include "struct.h"
#include "data.h"
#include "defs.h"
#include "sound.h"
#include "prize.h"
#include "player.h"
#include "draw.h"
#include "point.h"
#include "aliens.h"
#include "torps.h"

int num_curve_angles = 128;

static struct prize_map prize_maps[NUMPRIZES] =
{
  {I_PR_SING, "Increases weapon by small amount", BONUS},
  {I_PR_DOUB, "Increases weapon by medium amount", BONUS},
  {I_PR_TRIP, "Increases weapon by large amount", BONUS},
  {I_PR_CLUSTER_SING, "Increases cluster weapon slightly", BONUS},
  {I_PR_CLUSTER_DOUB, "Increases cluster weapon more", BONUS},
  {I_PR_CLUSTER_TRIP, "Increases cluster weapon most", BONUS},
  {I_PR_WAVE_SING, "Increases wave weapon slightly", BONUS},
  {I_PR_WAVE_DOUB, "Increases wave weapon more", BONUS},
  {I_PR_WAVE_TRIP, "Increases wave weapon most", BONUS},
  {I_PR_TREE_SING, "Increases tree weapon slightly", BONUS},
  {I_PR_TREE_DOUB, "Increases tree weapon more", BONUS},
  {I_PR_TREE_TRIP, "Increases tree weapon most", BONUS},
  {I_PR_SPEED, "Increments speed", BONUS},
  {I_PR_SHIELD, "Invulnerability", BONUS},
  {I_PR_BRAIN, "Hurts all onscreen aliens", BONUS},
  {I_PR_LEMON, "Removes all ship upgrades", MALUS},
  {I_PR_EXTRABULLET, "Destroys all etorps", BONUS},
  {I_PR_WRAP, "Increases life of torps", BONUS},
  {I_PR_SAFETY_BUBBLE, "Protects from one hit", BONUS},
  {I_PR_INC_CANNON_SPREAD, "Increases weapon spread", NEUTRAL},
  {I_PR_DEC_CANNON_SPREAD, "Decreases weapon spread", NEUTRAL},
  {I_PR_SHOOT_DOWN_ETORPS, "Etorps can be shot down", BONUS},
  {I_PR_INC_TORP_SPEED, "Increments launch speed of torps", NEUTRAL},
  {I_PR_DEC_TORP_SPEED, "Decrements launch speed of torps", NEUTRAL},
  {I_PR_INC_PRIZE_SPEED, "Increments prize speed", NEUTRAL},
  {I_PR_DEC_PRIZE_SPEED, "Decrements prize speed", NEUTRAL},
  {I_PR_BLIZZARD1, "Emits a storm of torps", BONUS},
  {I_PR_BLIZZARD2, "Emits a heavier storm of torps", BONUS},
  {I_PR_CHANGER, "Changes every 60 frames", BONUS},
  {I_PR_BONUS_ATTRACTOR, "Attracts bonuses", BONUS},
  {I_PR_MALUS_ATTRACTOR, "Attracts maluses", MALUS},
  {I_PR_NEUTRAL_ATTRACTOR, "Attracts neutrals", NEUTRAL},
  {I_PR_BONUS_REPULSOR, "Repulses bonuses", MALUS},
  {I_PR_MALUS_REPULSOR, "Repulses maluses", BONUS},
  {I_PR_NEUTRAL_REPULSOR, "Repulses neutrals", NEUTRAL},
  {I_PR_BONUS_DESTRUCTOR, "Destroys bonuses", MALUS},
  {I_PR_MALUS_DESTRUCTOR, "Destroys maluses", BONUS},
  {I_PR_NEUTRAL_DESTRUCTOR, "Destroys neutrals", NEUTRAL}
};

W_Color prize_frame_colors[NUMCATS]=
{ RED, GREEN, CYAN};
W_Image *prizeImages[NUMPRIZES];

extern struct torp *first_etorp;
extern int numetorps;

prize_t *first_prize; /* 8-) */
int num_prizes = 0;


void init_prizes()
{
  prize_type_t pr;

  for (pr= 0 ; pr < NUMPRIZES ; ++pr)
  {
    prizeImages[pr] = getImage(prize_maps[pr].image);
  }
}

void show_prize_list(int list_start)
  {
    int prize_index;
    int prizes_per_list = NUMPRIZES/3;
    int spacing = 30;
    char buf[40];
    list_start *= prizes_per_list;
    for(prize_index = list_start; prize_index < list_start + prizes_per_list;
        prize_index++)
    {
      W_DrawImage(baseWin, WINWIDTH/2 - 100, 100+(prize_index-list_start)*spacing, 0
        ,prizeImages[prize_index], W_Cyan);
      sprintf(buf, prize_maps[prize_index].description);
      mask_text_wrapper(baseWin, WINWIDTH/2 - 70
        , 100+(prize_index-list_start)*spacing, W_Yellow, buf, strlen(buf)
        , W_RegularFont);
    }
  }

void new_prize(point_t location)
{
    prize_t *p;

    num_prizes++;
    p = malloc(sizeof(prize_t));
    p->next = first_prize;
    p->prev = 0;
    if(first_prize)
    {
      first_prize->prev = p;
    }
    first_prize = p;
    p->location = location;
    p->type = get_random(NUMPRIZES);
    p->life_left = 1000;
    p->life_left*= (1 + (p->type == PR_LEMON));
#ifdef NO_USELESS_PRIZES
    while(p->type == PR_INC_CANNON_SPREAD || p->type == PR_DEC_CANNON_SPREAD
      || p->type == PR_INC_TORP_SPEED || p->type == PR_DEC_TORP_SPEED
      || p->type == PR_INC_PRIZE_SPEED || p->type == PR_DEC_PRIZE_SPEED
      || p->type == PR_BONUS_ATTRACTOR || p->type == PR_MALUS_ATTRACTOR
      || p->type == PR_NEUTRAL_ATTRACTOR || p->type == PR_BONUS_REPULSOR
      || p->type == PR_MALUS_REPULSOR || p->type == PR_NEUTRAL_REPULSOR
      || p->type == PR_BONUS_DESTRUCTOR || p->type == PR_MALUS_DESTRUCTOR
      || p->type == PR_NEUTRAL_DESTRUCTOR)
    {
      p->type = get_random(NUMPRIZES);
    }
#endif // NO_USELESS_PRIZES
    if(p->type == PR_CHANGER)
    {
      p->time_to_change = 30;
      while(p->type == PR_CHANGER)
      {
        p->type = get_random(NUMPRIZES);
      }
    }
    else
    {
      p->time_to_change = -1;
    }
    p->category = prize_maps[p->type].category;
    p->move_curve.current_dir = get_random(num_curve_angles);
    p->move_curve.end_dir = get_random(num_curve_angles);
    p->move_curve.steer_dir = get_random(2);
    if(p->move_curve.steer_dir == 0)
    {
      p->move_curve.steer_dir--;
    }
    /*
    printf("Prize %02d has current_dir of %01d, end_dir of %01d, and steer_dir of %01d\n"
      , num_prizes, p->move_curve.current_dir, p->move_curve.end_dir, p->move_curve.steer_dir);
    */
}

point_t vector_to_player(point_t location, double speed)
  {
    /* I suspect I was doing something subtly wrong with x_dir_to_player so I'm
       writing a new function that handles x and y */
    /* Get the distorted player locations. */
    /*
    point_t p_location1 = {player_0.location.x + SPACE_X_SIZE, player_0.location.y};
    point_t p_location2 = {player_0.location.x + SPACE_X_SIZE
                           , player_0.location.y + SPACE_Y_SIZE};
    point_t p_location3 = {player_0.location.x, player_0.location.y + SPACE_Y_SIZE};
    point_t p_location4 = {player_0.location.x - SPACE_X_SIZE
                           , player_0.location.y + SPACE_Y_SIZE};
    point_t p_location5 = {player_0.location.x - SPACE_X_SIZE, player_0.location.y};
    point_t p_location6 = {player_0.location.x - SPACE_X_SIZE
                           , player_0.location.y - SPACE_Y_SIZE};
    point_t p_location7 = {player_0.location.x, player_0.location.y - SPACE_Y_SIZE};
    point_t p_location8 = {player_0.location.x + SPACE_X_SIZE
                           , player_0.location.y - SPACE_Y_SIZE};
    */
    /* Get the vectors to the different locations. */
    /* a - b gives a vector to a */
    point_t real_diff = point_subtract(player_0.location, location);
    /*
    point_t diff1 = point_subtract(p_location1, location);
    point_t diff2 = point_subtract(p_location2, location);
    point_t diff3 = point_subtract(p_location3, location);
    point_t diff4 = point_subtract(p_location4, location);
    point_t diff5 = point_subtract(p_location5, location);
    point_t diff6 = point_subtract(p_location6, location);
    point_t diff7 = point_subtract(p_location7, location);
    point_t diff8 = point_subtract(p_location8, location);
    
    /* Get the angle to that one. 
    double angle = atan2(chosen_diff.y, chosen_diff.x) + M_PI;
    /* Get the speed modified vector for that angle. 
    point_t vector = {cos(angle) * speed, sin(angle) * speed};
    if(vector.x > 0 && vector.x > chosen_diff.x)
    */
    /* See which one is closest. */
    point_t chosen_diff = real_diff;

    point_t vector= unit_vector(chosen_diff);
    vector= scalar_multiply(vector,speed);
    if(vector.x > 0 && vector.x > chosen_diff.x)
    {
      vector.x = chosen_diff.x;
    }
    if(vector.x < 0 && vector.x < chosen_diff.x)
    {
      vector.x = chosen_diff.x;
    }
    if(vector.y > 0 && vector.y > chosen_diff.y)
    {
      vector.y = chosen_diff.y;
    }
    if(vector.y < 0 && vector.y < chosen_diff.y)
    {
      vector.y = chosen_diff.y;
    }
    return vector;
  }

void do_prizes()
{
    prize_t *p = first_prize, *nextp;
    prize_t *prize = first_prize; /* used by the loops in the change prize velocity prizes */
    player_t *player_p = &player_0;
    int i;
#ifdef SOUND
    int oldPlaySounds;
#endif

    while(p)
    {
      nextp = p->next;
	if(!paused)
        {
          /*
          if(p->speed.y == 0)
             p->speed.y = get_random(5) - 5/2;
          if(p->speed.x == 0)
             p->speed.x = get_random(5) - 5/2;
          */          
          p->life_left-=num_prizes;
#if defined (SEEKING_LEMONS)
          if(p->type==PR_LEMON)
          {
            p->speed = vector_to_player(p->location, 5);
          }
#endif /* SEEKING_LEMONS */
          
          if(p->move_curve.current_dir == p->move_curve.end_dir)
          {
            p->move_curve.end_dir = get_random(num_curve_angles);
            p->move_curve.steer_dir = get_random(2);
            if(p->move_curve.steer_dir == 0)
            {
              p->move_curve.steer_dir--;
            }            
          }
          p->speed.x = (cos((p->move_curve.current_dir * M_PI)/(num_curve_angles/2))
            * player_0.prize_speed_range);
          p->speed.y = (-sin((p->move_curve.current_dir * M_PI)/(num_curve_angles/2))
            * player_0.prize_speed_range);
          p->move_curve.current_dir += p->move_curve.steer_dir;
          if(p->move_curve.current_dir < 0)
          {
            p->move_curve.current_dir += num_curve_angles;
          }
          if(p->move_curve.current_dir > num_curve_angles)
          {
            p->move_curve.current_dir -= num_curve_angles;
          }
          
          /*
          printf("Prize %02d has current_dir of %01d, end_dir of %01d, and steer_dir of %01d\n"
            , num_prizes, p->move_curve.current_dir, p->move_curve.end_dir, p->move_curve.steer_dir);
          */
          //subtract_V2(&p->x_speed, &p->y_speed, player_0.x, player_0.y, p->x, p->y);
          //subtract_V2(&p->x_speed, &player_0.location, &p->location);
          p->speed = point_add(vector_to_player(p->location
                                                , player_0.attractions[p->category])
                               , p->speed);
          p->location = point_add(p->speed, p->location);
        }
        wrap_object(&p->location);

	if((!player_0.dead)
          && ((p->location.x)>(player_0.location.x-(playerShip->width))
            && (p->location.x)<(player_0.location.x+(playerShip->width)))
          && ((p->location.y)>(player_0.location.y-(playerShip->height))
            && (p->location.y)<(player_0.location.y+(playerShip->height)))
          && (p->life_left>0))
        {
	    p->life_left = 0;
#ifdef SOUND
	    play_sound(SND_DDLOO);
#endif
            if(p->type != PR_LEMON)
            {
              player_0.got_lemon=0;
            }
	    switch(p->type) {
	      case PR_SING:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[0] += 1 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[0]=
                    (player_0.weapons[0] < 2 ? player_0.weapons[0] + 1
                        :sqrt(sqr(player_0.weapons[0]) + 2)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
		break;
	      case PR_DOUB:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[0] += 2 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[0]=
                    (player_0.weapons[0] < 3 ? player_0.weapons[0] + 2
                        :sqrt(sqr(player_0.weapons[0]) + 4)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
		break;
              case PR_TRIP:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[0] += 3 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[0]=
                    (player_0.weapons[0] < 4 ? player_0.weapons[0] + 3
                        :sqrt(sqr(player_0.weapons[0]) + 6)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
		break;
                
              case PR_CLUSTER_SING:
                player_0.cluster_bombs += 1 * WEAPON_FACTOR;
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[1] += 1 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[1] =
                    (player_0.weapons[1] < 2 ? player_0.weapons[1] + 1
                        :cbrt(cube(player_0.weapons[1]) + 2)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                break;
              case PR_CLUSTER_DOUB:
                player_0.cluster_bombs+=2 * WEAPON_FACTOR;
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[1] += 2 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[1] =
                    (player_0.weapons[1] < 3 ? player_0.weapons[1] + 2
                        :cbrt(cube(player_0.weapons[1]) + 4)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                break;
              case PR_CLUSTER_TRIP:
                player_0.cluster_bombs+=3 * WEAPON_FACTOR;
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[1] += 3 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[1] =
                    (player_0.weapons[1] < 4 ? player_0.weapons[1] + 3
                        :cbrt(cube(player_0.weapons[1]) + 6)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                break;

              case PR_WAVE_SING:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[2] += 1 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[2] =
                    (player_0.weapons[2] < 2 ? player_0.weapons[2] + 1
                      :sqrt(sqr(player_0.weapons[2]) + 2)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                if(player_0.weapons[2] >= player_0.wave_amplitude * 2)
                {
                  player_0.wave_amplitude *= 2;
                }
                break;
              case PR_WAVE_DOUB:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[2] += 2 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[2] =
                (player_0.weapons[2] < 3 ? player_0.weapons[2] + 2
                  :sqrt(sqr(player_0.weapons[2]) + 4)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                if(player_0.weapons[2] >= player_0.wave_amplitude * 2)
                {
                  player_0.wave_amplitude *= 2;
                }
                break;
              case PR_WAVE_TRIP:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[2] += 3 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[2] =
                (player_0.weapons[2] < 4 ? player_0.weapons[2] + 3
                  :sqrt(sqr(player_0.weapons[2]) + 6)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                if(player_0.weapons[2] >= player_0.wave_amplitude * 2)
                {
                  player_0.wave_amplitude *= 2;
                }
                break;

              case PR_TREE_SING:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[3] += 1 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[3] =
                    (player_0.weapons[3] < 2 ? player_0.weapons[3] + 1
                      :sqrt(sqr(player_0.weapons[3]) + 2)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                break;
              case PR_TREE_DOUB:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[3] += 2 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[3] =
                    (player_0.weapons[3] < 3 ? player_0.weapons[3] + 2
                      :sqrt(sqr(player_0.weapons[3]) + 4)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                break;
              case PR_TREE_TRIP:
#ifdef INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[3] += 3 * WEAPON_FACTOR;
#else // INCREMENTAL_WEAPON_INCREASE
                player_0.weapons[3] =
                    (player_0.weapons[3] < 4 ? player_0.weapons[3] + 3
                      :sqrt(sqr(player_0.weapons[3]) + 6)) * WEAPON_FACTOR;
#endif // INCREMENTAL_WEAPON_INCREASE
                break;

	      case PR_SPEED:
		if(player_0.acceleration < MAXSPEED)
		    player_0.acceleration++;
                if(player_0.movespeed < MAXSPEED)
                player_0.movespeed++;
		break;
	      case PR_SHIELD:
		player_0.shield = SHIELDTIME;
#ifdef SOUND
		play_sound(SND_SHIELD);
#endif
		break;
	      case PR_SMART:
#if !defined(ALLOW_SMART_BOMB)                
#ifdef SOUND
		play_sound(SND_SMART);
		oldPlaySounds = playSounds;
		playSounds = 0;
#endif
		for(i=0;i<MAXALIENS;i++)
                {
                  switch(aliens[i].state)
                  {
                    case WAITING_TO_ENTER:
                      /* Ha Ha. Aliens that haven't entered yet don't
                         get hurt.
                      */
                      break;
                    case ENTERING:
                    case IN_CONVOY:
                    case ATTACKING:
                    case ESCORTING:
                    case RETURNING_TO_CONVOY:
                      torp_hit_alien(aliens+i);
                    case DEAD:
                      /* Dead ones don't get hurt either.*/
                      break;
                    default:
                      fprintf(OUT_FILE,"Alien %i in unknown state %i!\n",i,aliens[i].state);
                  }
		}
#ifdef SOUND
		playSounds = oldPlaySounds;
#endif
#endif /* !defined(ALLOW_SMART_BOMB)                */
		break;
                
	      case PR_LEMON:
		player_0.got_lemon++;
                player_0.current_weapon= 0;
		player_0.weapons[0] = 1;
                player_0.weapons[1] = 0;
                player_0.weapons[2] = 0;
                player_0.weapons[3] = 0;
		player_0.acceleration = MINSPEED;
                player_0.movespeed = MINSPEED;
                player_0.safety_bubbles= 0;
                player_0.cannon_spread= 1;
                player_0.wave_amplitude = 8;
                player_0.shoot_down_etorps= 0;
                player_0.prize_speed_range=3;
                player_0.torp_velocity=0;
                player_0.cluster_bombs=0;
                player_0.torp_life = (TORPSPEED + sqrt(TORPSPEED + 8*SPACE_Y_MAX))/2;
                player_0.attractions[0] = 0;
                player_0.attractions[1] = 0;
                player_0.attractions[2] = 0;
		break;
                
	      case PR_EXTRABULLET:
                /* Destroy all etorps.*/
                {
                  torp_t* current_torp= first_etorp;
                  if (current_torp)
                  {
                    do
                    {
                      current_torp->alive= 0;
                      current_torp= current_torp->next;
                      /*Remember, etorps is a circular linked list.*/
                    }while (current_torp && current_torp != first_etorp);
                  }
                }
		break;

              case PR_WRAP:
                player_0.torp_life+=(TORPSPEED + sqrt(TORPSPEED + 8*SPACE_Y_MAX))/2;
                break;

              case PR_SAFETY_BUBBLE:
                ++player_0.safety_bubbles;
                break;

              case PR_INC_CANNON_SPREAD:
                player_0.cannon_spread*=1.5;
                break;

              case PR_DEC_CANNON_SPREAD:
                player_0.cannon_spread/=1.5;
                break;

              case PR_SHOOT_DOWN_ETORPS:
                ++player_0.shoot_down_etorps;
                ++player_0.torp_velocity;
                break;

              case PR_INC_TORP_SPEED:
                ++player_0.torp_velocity;
                break;
                
              case PR_DEC_TORP_SPEED:
                --player_0.torp_velocity;
                break;

              case PR_INC_PRIZE_SPEED:
                while(prize)
                {
                  if(prize->speed.x<0)
                  prize->speed.x-=1;
                  else
                  prize->speed.x+=1;
                  if(prize->speed.y<0)
                  prize->speed.y-=1;
                  else
                  prize->speed.y+=1;
                  prize=prize->next;
                }
                ++player_0.prize_speed_range;
                break;

              case PR_DEC_PRIZE_SPEED:
                while(prize)
                {
                  if(prize->speed.x<0)
                  prize->speed.x+=1;
                  else
                  prize->speed.x-=1;
                  if(prize->speed.y<0)
                  prize->speed.y+=1;
                  else
                  prize->speed.y-=1;
                  prize=prize->next;
                }
                if(player_0.prize_speed_range>1)
                --player_0.prize_speed_range;
                break;
              case PR_BLIZZARD1:
                /*
                  lauch the torps to the right.
                */
                spread_torps(player_p->location
                  , (TORPSPEED + sqrt(TORPSPEED + 8*SPACE_Y_MAX))/2*2
                  , TORPSPEED+player_p->torp_velocity
                  , 1,  M_PI_2, M_PI/25.0
                  , 50 * WEAPON_FACTOR, 0, 0);
                /*
                  lauch the torps to the left.
                */
                spread_torps(player_p->location
                  , (TORPSPEED + sqrt(TORPSPEED + 8*SPACE_Y_MAX))/2*2
                  , TORPSPEED+player_p->torp_velocity
                  , 1,  M_PI_2, -M_PI/25.0
                  , 50 * WEAPON_FACTOR, 0, 0);
                break;
              case PR_BLIZZARD2:
                /*
                  lauch the torps to the right.
                */
                spread_torps(player_p->location
                  , (TORPSPEED + sqrt(TORPSPEED + 8*SPACE_Y_MAX))/2*4
                  , TORPSPEED+player_p->torp_velocity
                  , 1,  M_PI_2, M_PI/25.0
                  , 100 * WEAPON_FACTOR, 0, 0);
                /*
                  lauch the torps to the left.
                */
                spread_torps(player_p->location
                  , (TORPSPEED + sqrt(TORPSPEED + 8*SPACE_Y_MAX))/2*4
                  , TORPSPEED+player_p->torp_velocity
                  , 1,  M_PI_2, -M_PI/25.0
                  , 100 * WEAPON_FACTOR, 0, 0);
                break;
                
              case PR_BONUS_ATTRACTOR:
                player_0.attractions[BONUS] += 2;
                break;

              case PR_MALUS_ATTRACTOR:
                player_0.attractions[MALUS] += 2;
                break;

              case PR_NEUTRAL_ATTRACTOR:
                player_0.attractions[NEUTRAL] += 2;
                break;

              case PR_BONUS_REPULSOR:
                player_0.attractions[BONUS] -= 2;
                break;

              case PR_MALUS_REPULSOR:
                player_0.attractions[MALUS] -= 2;
                break;

              case PR_NEUTRAL_REPULSOR:
                player_0.attractions[NEUTRAL] -= 2;
                break;

              case PR_BONUS_DESTRUCTOR:
                while(prize)
                {
                  if(prize->category == BONUS)
                  {
                    prize->life_left = 0;
                  }
                  prize=prize->next;
                }
                break;

              case PR_MALUS_DESTRUCTOR:
                while(prize)
                {
                  if(prize->category == MALUS)
                  {
                    prize->life_left = 0;
                  }
                  prize=prize->next;
                }
                break;

              case PR_NEUTRAL_DESTRUCTOR:
                while(prize)
                {
                  if(prize->category == NEUTRAL)
                  {
                    prize->life_left = 0;
                  }
                  prize=prize->next;
                }
                break;

              case PR_CHANGER:

                break;
                
              default:
                fprintf(OUT_FILE,"Prize of  unknown type %i!\n",p->type);
	    }
	} 
        else
        {
          if(p->time_to_change > 0)
          {
            --p->time_to_change;
          }
          if (p->time_to_change == 0 || (p->type == PR_CHANGER
                && p->time_to_change == -1))
          {
            if (!get_random(10))
            {
              p->type= get_random(NUMPRIZES);
              p->time_to_change = 30;
            }
          }
        }
        if(p->life_left<1)
        {
          if(p->next)
          {
            p->next->prev = p->prev;
          }
          if(p->prev)
          {
            p->prev->next = p->next;
          }
          if(p == first_prize)
          {
            first_prize = p->next;
          }
          free(p);
          --num_prizes;
        }
	p=nextp;
    }
}

/*
void clean_up_prizes(void)
  {
     prize_t *p = first_prize, *nextp;
     if(!p) return;
     nextp = p->next;
     while(p)
     {
       cache_clear_area_wrapper
       (
         baseWin
         , ((int)p->location.x)-prizeImages[p->type]->width/2
         , ((int)p->location.y)-prizeImages[p->type]->width/2,
         prizeImages[p->type]->width+1,
         prizeImages[p->type]->height
       );
       if(p->next)
       {
         p->next->prev = p->prev;
       }
       if(p->prev)
       {
         p->prev->next = p->next;
       }
       if(p == first_prize)
       {
         first_prize = p->next;
       }
       if(p != NULL)
         free(p);
       p=nextp;
     }
     num_prizes = 0;
  }
*/
