#ifndef __STRUCT_H__
#define __STRUCT_H__
#include "libsprite/Wlib.h"
#include "defs.h"

struct point
{
  double x, y;
};

typedef struct point point_t;

struct directional_curve
{
  int current_dir, end_dir, steer_dir;
};

struct torp
{
  struct torp *next, *prev;
  int alive, frame;
  point_t location;
  point_t speed;
  point_t accel;
  point_t original_speed;
  int cluster_bombs;
  int fluctuate;
  int hits;
  double wave;
  double amplitude;
  int branches;
  double angle;
};

typedef struct torp torp_t;

struct torp_prop
{
  int numtorps;
  int* next_free_torp_id;
  int id_of_free_torp_id_pointed_to;
  int free_torp_ids[MAXTORPS];
};

typedef struct torp_prop torp_p;

struct star
{
    point_t location;
    double speed;
    W_Color color;
};

typedef struct star star_t;

struct blood
{
  point_t location;
  point_t speed;
  W_Color color;
  int alive;
};

typedef struct blood blood_t;

enum prize_type
{
  PR_SING,  
  PR_DOUB,  
  PR_TRIP,
  PR_CLUSTER_SING,
  PR_CLUSTER_DOUB,
  PR_CLUSTER_TRIP,
  PR_WAVE_SING,
  PR_WAVE_DOUB,
  PR_WAVE_TRIP,
  PR_TREE_SING,
  PR_TREE_DOUB,
  PR_TREE_TRIP,
  PR_SPEED, 
  PR_SHIELD,
  PR_SMART, 
  PR_LEMON, 
  PR_EXTRABULLET,
  PR_WRAP,
  PR_SAFETY_BUBBLE,
  PR_INC_CANNON_SPREAD,
  PR_DEC_CANNON_SPREAD,
  PR_SHOOT_DOWN_ETORPS,
  PR_INC_TORP_SPEED,
  PR_DEC_TORP_SPEED,
  PR_INC_PRIZE_SPEED,
  PR_DEC_PRIZE_SPEED,
  PR_BLIZZARD1,
  PR_BLIZZARD2,
  PR_CHANGER,
  PR_BONUS_ATTRACTOR,
  PR_MALUS_ATTRACTOR,
  PR_NEUTRAL_ATTRACTOR,
  PR_BONUS_REPULSOR,
  PR_MALUS_REPULSOR,
  PR_NEUTRAL_REPULSOR,
  PR_BONUS_DESTRUCTOR,
  PR_MALUS_DESTRUCTOR,
  PR_NEUTRAL_DESTRUCTOR,
  NUMPRIZES,
};
typedef enum prize_type prize_type_t;

enum prize_category
{
  MALUS,
  BONUS,
  NEUTRAL,
  NUMCATS,
};
typedef enum prize_category prize_category_t;

struct prize
{
  struct prize *next, *prev;
  point_t location;
  point_t speed;
  int life_left;
  int time_to_change;
  prize_type_t type;
  struct directional_curve move_curve;
  enum prize_category category;
};
typedef struct prize prize_t;

/* Finally, all the different prize related maps are gathered together. */
struct prize_map
{
  int image;
  char* description;
  prize_category_t category;
};

enum alien_state
{
  DEAD=0,
  WAITING_TO_ENTER,
  ENTERING,
  IN_CONVOY,
  ATTACKING,
  ESCORTING,
  RETURNING_TO_CONVOY,
};

typedef enum alien_state alien_state_t;

struct alien
{
    point_t location;
    alien_state_t state;
    int dir;
    int steer;
    int alive;
    int dying;
    int path, path_pos;
    int escorting;
    int entering, enterdelay;
    int type;
    int id;
    W_Image *shape;
};

typedef struct alien alien_t;

struct explosion
{
    struct explosion *next, *prev;
    point_t location;
    int frame;
    W_Image *shape;
};

typedef struct explosion explosion_t;

struct score_bubble {
    struct score_bubble *next, *prev;
    point_t location;
    int count;
    W_Image *shape;
};

typedef struct score_bubble score_bubble_t;

struct game_key
{
  char chars;
  int flags;
  char description[32];
};

typedef struct game_key g_key_t;

enum foogle { number_of_keys= 14 };

struct player
{
  point_t location;
  point_t speed;
  int movespeed;
  double acceleration; /* acceleration is the amount by which the player can accelerate */
  double weapons[4];
  /* arc_weapon, cluster_weapon, wave_weapon, tree_weapon */
  int weapon_delays[4];
  int fire_times[4];
  /* Same as above. */
  int current_weapon;
  double wave_amplitude;
  int tree_branches;
  double cannon_spread;
  g_key_t keys[number_of_keys];
  int pressed_keys;
  int flash;
  int dead;
  int shield;
  int ships;
  int score;
  int last_score;
  int next_bonus;
  int got_lemon;
  int safety_bubbles;
  int shoot_down_etorps;
  int torp_velocity;
  int prize_speed_range;
  int cluster_bombs;
  int torp_life;
  int attractions[3];
};

typedef struct player player_t;

struct my_event
{
  char key;
  int type;
};

typedef struct my_event ev_t;
#endif /* __STRUCT_H__ */
