#include <math.h>
#include <stdio.h>

#include "struct.h"
#include "data.h"
#include "defs.h"
#include "torps.h"
#include "player.h"
#include "aliens.h"
#include "point.h"

torp_p torp_stuff;
torp_t torps[MAXTORPS];

void init_torps()
  {
    int i;
    torp_stuff.numtorps = 0;
    torp_stuff.id_of_free_torp_id_pointed_to = 0;
    torp_stuff.next_free_torp_id =
      &torp_stuff.free_torp_ids[torp_stuff.id_of_free_torp_id_pointed_to];
    for(i=0; i<MAXTORPS; i++)
    {
      torp_stuff.free_torp_ids[i] = i;
      torps[i].alive = 0;
    }
  }

void new_torp(point_t location, int life, point_t speed, point_t accel
  , int cluster_bombs, int fluctuate, double angle, double wave, double amplitude
  , int branches)
  {
    int i;
    /* Replaces loop!!!! 
    torp_stuff.id_of_free_torp_id_pointed_to++;
    torp_stuff.next_free_torp_id=
    &torp_stuff.free_torp_ids[torp_stuff.id_of_free_torp_id_pointed_to];
    */

    for(i= 0; i < MAXTORPS; i++)
    {
      if(!torps[i].alive)
      {
        torps[i].location = location;
        torps[i].alive = life;
        torps[i].speed = speed;
        torps[i].accel = accel;
        torps[i].original_speed.x = cos(angle);
        torps[i].original_speed.y = -sin(angle);
        torps[i].cluster_bombs= cluster_bombs;
        torps[i].fluctuate = fluctuate;
        torps[i].hits = 1;
        torps[i].wave = wave;
        torps[i].amplitude = amplitude;
        torps[i].branches = branches;
        torps[i].angle = angle;
        torp_stuff.numtorps++;
#ifdef SOUND
        play_sound(SND_FIRETORP);
#endif
        return;
      }
    }
  }

void kill_torp(int id)
  {
    torps[id].alive = 0;
    --torp_stuff.numtorps;
    /*
    --torp_stuff.id_of_free_torp_id_pointed_to;
    torp_stuff.next_free_torp_id=
      &torp_stuff.free_torp_ids[torp_stuff.id_of_free_torp_id_pointed_to];
    torp_stuff.free_torp_ids[torp_stuff.id_of_free_torp_id_pointed_to] = id;
    */
  }

void wave_torp(int i)
  {
    if(torps[i].amplitude)
    {
      torps[i].speed.x = cos(torps[i].wave) * torps[i].amplitude;
      torps[i].wave -= M_PI/8;
    }
  }

void flux_torp(int i)
  {
    if(torps[i].fluctuate)
    {
      if(torps[i].alive%2)
      {
        torps[i].speed.x = torps[i].original_speed.x
        * sin(torps[i].fluctuate / 5) * 10;
        torps[i].speed.y = torps[i].original_speed.y
        * sin(torps[i].fluctuate / 5) * 10;
      }
      torps[i].fluctuate--;
    }
    else
    {
      torps[i].speed.x += torps[i].accel.x;
      torps[i].speed.y += torps[i].accel.y;
    }
  }

void cluster_torp(int i)
  {
    if(torps[i].cluster_bombs)
    {
      double start_angle=  M_PI_2;
      double inc_angle= M_PI/(torps[i].cluster_bombs*2);
      /*
        lauch the torps to the left.
      */
      spread_torps(torps[i].location
        , (TORPSPEED + sqrt(TORPSPEED + 8*SPACE_Y_MAX))/2*3
        , TORPSPEED+player_0.torp_velocity
        , 1, start_angle+inc_angle, inc_angle
        , torps[i].cluster_bombs*2
        , torps[i].cluster_bombs - 1, 100);
      /*
        lauch the torps to the right.
      */
      spread_torps(torps[i].location
        , (TORPSPEED + sqrt(TORPSPEED + 8*SPACE_Y_MAX))/2*3
        , TORPSPEED+player_0.torp_velocity
        , 1, start_angle, -inc_angle
        , torps[i].cluster_bombs*2
        , torps[i].cluster_bombs - 1, 100);
    }
  }

int alien_check_torp(int i)
  {
    int j;
    for(j=0;j<MAXALIENS;j++)
    {
      if((aliens[j].alive) && (!aliens[j].dying) &&
        (aliens[j].state!=WAITING_TO_ENTER)&&
        (ABS(torps[i].location.x - aliens[j].location.x) < 8) &&
        ((ABS(torps[i].location.y-aliens[j].location.y) < 8) ||
          (ABS((torps[i].location.y+torps[i].speed.y/2)
            -aliens[j].location.y) < 8)))
      {
        torp_hit_alien(&aliens[j]);
        cluster_torp(i);
        torps[i].hits--;
        if(torps[i].hits<=0)
        {
          kill_torp(i);
          return 1;
        }
      }
    }
    return 0;
  }

void torp_wave(point_t location, int life, double amplitude, int torps)
  {
    int i;
    double wave = (56 * M_PI) / 8;
    double wave_change = M_PI / 8;
    double x_diff_between_torps = amplitude / torps;
    point_t speed = {sin(wave) * amplitude, TORPSPEED + player_0.torp_velocity};
    point_t accel = {0, -1};
    for(i = 0; i < torps; i++)
    {
      speed.x = sin(wave) * amplitude;
      location.x += x_diff_between_torps;
      new_torp(location, life, speed, accel, 0, 0, M_PI_2, wave, amplitude, 0);
      wave *= -1;
      (wave > 0) ? (wave -= wave_change) : (wave += wave_change);
    }
  }

void torp_line(point_t location, int life, double base_torp_speed
  , double seperation, int torps, int cluster_bombs , double angle, int branches)
  {
    int i;
    point_t speed = {cos(angle) * base_torp_speed, -sin(angle) * base_torp_speed};
    point_t accel = {0, 0};
    if(!branches)
    {
      accel.x = cos(angle);
      accel.y = -sin(angle);
    }
    double angular_seperation = M_PI_2;
    int linear_seperation = seperation;
    /* Get locations of torps to either side */
    point_t location_1 = {location.x + cos(angle - angular_seperation)
                          * linear_seperation
                          , location.y - sin(angle - angular_seperation)
                          * linear_seperation};
    point_t location_2 = {location.x + cos(angle + angular_seperation)
                          * linear_seperation
                          , location.y - sin(angle + angular_seperation)
                          * linear_seperation};
    /* Get slope from one torp location to the next */
    point_t slope = point_subtract(location_1, location_2);
    /* Get leftmost torp location */
    point_t start = point_subtract(location, scalar_multiply(slope
                  , ((torps % 2) ? floor(torps / 2) : ((double)(torps - 1) / 2.0))));
    /* Start firing from there */
    for(i = 0; i < torps; i++)
    {
      new_torp(point_add(start, scalar_multiply(slope, i)), life, speed, accel
        , cluster_bombs, 0, angle, 0, 0, branches);
    }
  }

void torp_tree(point_t location, int life, int torps, int branches, double angle)
  {
    int i;
    point_t speed;
    point_t accel = {0, 0}; // I don't want the tree torps to accelerate.
    double inc_angle = M_PI/16.0 * player_0.cannon_spread;
    /* Fire center */
    torp_line(location, life, 1, 9, (!(torps % 2)) + 1, 0, angle, branches - 1);
    torps -= (!(torps % 2)) + 1;
    /* Fire left */
    for(i = 0; i < torps / 2; i++)
    {
      speed.x = cos(angle - (inc_angle * i));
      /* y speed is negative because y increases going down. */
      speed.y = -sin(angle - (inc_angle * i));
      new_torp(location, life, speed, accel, 0, 0, angle - (inc_angle * i), 0, 0
        , branches - 1);
    }
    /* Fire right */
    for(i = 0; i < torps / 2; i++)
    {
      speed.x = cos(angle + (inc_angle * i));
      speed.y = -sin(angle + (inc_angle * i));
      new_torp(location, life, speed, accel, 0, 0, angle + (inc_angle * i), 0, 0
        , branches - 1);
    }
  }

int branch_torp(int i)
  {
    if(torps[i].branches)
    {
      torp_tree(torps[i].location, player_0.torp_life
        , torps[i].branches, torps[i].branches
        , torps[i].angle);
      /* Kill it because it branched. */
      kill_torp(i);
      return 1;
    }
    return 0;
  }

void spread_torps(point_t location, int life, double base_torp_speed
  , double base_torp_accel, double start_angle, double inc_angle
  , int torps, int cluster_bombs, int fluctuate)
  {
    int current;
    point_t torp_speed;
    point_t torp_accel;
    double angle;
    for (current= 0, angle= start_angle; current < torps
         ; ++current, angle+= inc_angle)
    {
      torp_speed.x= cos(angle)*base_torp_speed;
      torp_accel.x= cos(angle)*base_torp_accel;
      /* The y speed is negative because y increases going *down*. */
      torp_speed.y= -sin(angle)*base_torp_speed;
      torp_accel.y= -sin(angle)*base_torp_accel;
      //fprintf(OUT_FILE, "In %s, line %d, file %s : torp_x_speed == %f, torp_y_speed == %f\n"
      //  , __PRETTY_FUNCTION__, __LINE__, __FILE__, torp_x_speed, torp_y_speed );
      new_torp(location, life, torp_speed, torp_accel, cluster_bombs, fluctuate
        , angle, 0, 0, 0);
    }
  }

void do_torps()
  {
    int i;
    for(i=0;i<MAXTORPS;i++)
    {
      /* Don't play with dead things, it's bad for your health. */
      if(torps[i].alive)
      {
        if(!paused)
        {
          if(branch_torp(i))
          {
            continue;
          }
          wave_torp(i);
          flux_torp(i);
          torps[i].location.y += torps[i].speed.y;
          torps[i].location.x += torps[i].speed.x;
          torps[i].frame++;
          if(alien_check_torp(i))
          {
            continue;
          }
          wrap_object(&torps[i].location);
          --torps[i].alive;
          if(torps[i].alive<=0)
          {
            kill_torp(i);
            continue;
          }
        }
        /* Do no processing if paused.*/
      }
    }
  }

void fire_torps2(player_t* player_p)
  {
    int torps_to_launch;
    int torps_straight_up;
    int torps_spread_to_each_side;
    double base_torp_speed= TORPSPEED+player_p->torp_velocity;
    double angle;
    point_t torp_speed;
    double spread_angle= M_PI/25.0*player_p->cannon_spread;
    point_t torp_accel = {0, -1};
    
    torps_to_launch= (int)player_p->weapons[0];
    //fprintf(OUT_FILE, "In %s, line %d, file %s : torps_to_launch == %d\n", __PRETTY_FUNCTION__, __LINE__, __FILE__, torps_to_launch );

    /*
      If we are launching an even number of torps, we want 2 torps to go straight up.
      If we are launching an odd number of torps, we want 1 torp to go straight up.
      % == remainder operator. Divide by 2 and return the remainder.
     */
    if (torps_to_launch%2 == 0) 
    {
      torps_straight_up= 2; /*even*/
    }
    else
    {
      torps_straight_up= 1; /*odd*/
    }

    //fprintf(OUT_FILE, "In %s, line %d, file %s : torps_straight_up == %d\n", __PRETTY_FUNCTION__, __LINE__, __FILE__, torps_straight_up );
    /*
      Launch the torps that go straight up.
     */
    angle= M_PI_2; /* pi/2 radians is straight up.*/
    torp_speed.x = cos(angle)*base_torp_speed;
    /* The y speed is negative because y increases going *down*. */
    torp_speed.y = -sin(angle)*base_torp_speed;
    switch (torps_straight_up)
    {
      case 1:
        new_torp( player_p->location, player_p->torp_life, torp_speed, torp_accel, 0
          , 0, M_PI_2, 0, 0, 0);
        break;
      case 2:
        torp_line(player_p->location, player_p->torp_life, base_torp_speed, 6, 2, 0
          , M_PI_2, 0);
        break;
    }

    torps_spread_to_each_side= (torps_to_launch - torps_straight_up)/2;

    //fprintf(OUT_FILE, "In %s, line %d, file %s : torps_spread_to_each_side == %d\n", __PRETTY_FUNCTION__, __LINE__, __FILE__, torps_spread_to_each_side );
    if (torps_spread_to_each_side <= 0)
    {
      return;
    }
    /*
      lauch the torps to the right.
      IF you need to change the spread_angle, do it before spread_torps is called.
     */
    spread_torps(player_p->location, player_p->torp_life
      , TORPSPEED+player_p->torp_velocity, 1, angle+spread_angle, spread_angle
      , torps_spread_to_each_side, 0, 0);
    /*
      lauch the torps to the left.
     */
    spread_torps(player_p->location, player_p->torp_life
      , TORPSPEED+player_p->torp_velocity, 1, angle-spread_angle, -spread_angle
      , torps_spread_to_each_side, 0, 0);
  }
